@@ -1,8 +1,31 @@
-/* $XConsortium: Box.c,v 1.45 90/12/31 10:22:09 gildea Exp $ */
+/* $Xorg: Box.c,v 1.4 2001/02/09 02:03:43 xorgcvs Exp $ */
 
-/***********************************************************
-Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
-and the Massachusetts Institute of Technology, Cambridge, Massachusetts.
+/***********************************************************************
+
+Copyright 1987, 1988, 1994, 1998  The Open Group
+
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of The Open Group shall not be
+used in advertising or otherwise to promote the sale, use or other dealings
+in this Software without prior written authorization from The Open Group.
+
+
+Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
@@ -10,7 +33,7 @@ Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, 
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in 
-supporting documentation, and that the names of Digital or MIT not be
+supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
 software without specific, written prior permission.  
 
@@ -22,9 +45,9 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
-Usage of color converter for background color added: Roland Krause 1997
+This file contains modifications for XawPlus, Roland Krause 2002
 
-******************************************************************/
+*************************************************************************/
 
 
 /* 
@@ -146,12 +169,19 @@ static void DoLayout(bbw, width, height, reply_width, reply_height, position)
     Dimension lw, lh;	/* Width and height needed for current line 	*/
     Dimension bw, bh;	/* Width and height needed for current widget 	*/
     Dimension h_space;  /* Local copy of bbw->box.h_space 		*/
-    register Widget widget;	/* Current widget 			*/
+    Widget widget;	/* Current widget	 			*/
     int num_mapped_children = 0;
  
     /* Box width and height */
     h_space = bbw->box.h_space;
-    w = h_space;
+
+    w = 0;
+    for (i = 0; i < bbw->composite.num_children; i++) {
+	if ( bbw->composite.children[i]->core.width > w )
+            w = bbw->composite.children[i]->core.width;
+    }
+    w += h_space;
+    if ( w > width ) width = w;
     h = bbw->box.v_space;
    
     /* Line width and height */
@@ -209,6 +239,7 @@ static void DoLayout(bbw, width, height, reply_width, reply_height, position)
     if (!vbox && width && lw > width && lh < height) {
 	/* reduce width if too wide and height not filled */
 	Dimension width_needed = lw, sw = lw, sh = lh;
+
 	XtOrientation orientation = bbw->box.orientation;
 	bbw->box.orientation = XtorientVertical;
 	while (sh < height && sw > width) {
@@ -223,13 +254,17 @@ static void DoLayout(bbw, width, height, reply_width, reply_height, position)
 	}
 	bbw->box.orientation = orientation;
     }
-
+   if ( vbox && ( ( width < w ) || ( width < lw ) ) ) {
+        AssignMax(w, lw);
+        DoLayout( bbw, w, height, reply_width, reply_height, position );
+        return;
+    }
     if (position && XtIsRealized((Widget)bbw)) {
 	if (bbw->composite.num_children == num_mapped_children)
 	    XMapSubwindows( XtDisplay((Widget)bbw), XtWindow((Widget)bbw) );
 	else {
 	    int i = bbw->composite.num_children;
-	    register Widget *childP = bbw->composite.children;
+	    Widget *childP = bbw->composite.children;
 	    for (; i > 0; childP++, i--)
 		if (XtIsRealized(*childP) && XtIsManaged(*childP) &&
 		    (*childP)->core.mapped_when_managed)
@@ -566,7 +601,7 @@ Cardinal *num_args;
 } /* Initialize */
 
 static void Realize(w, valueMask, attributes)
-    register Widget w;
+    Widget w;
     Mask *valueMask;
     XSetWindowAttributes *attributes;
 {
@@ -577,7 +612,7 @@ static void Realize(w, valueMask, attributes)
 		    *valueMask, attributes);
 } /* Realize */
 
-
+/* ARGSUSED */
 static Boolean SetValues(current, request, new, args, num_args)
 Widget current, request, new;
 ArgList args;
