@@ -6,7 +6,21 @@
  * This widget adds a backing store to the simple widget and defines
  * all(?) drawing functions known from the Xlib for this widget.
  *
- * Author : Roland Krause, germany, summer 1996 + 2000
+ * Copyright (c) Roland Krause 2002, roland_krause@freenet.de
+ *
+ * This module is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  ***********************************************************************/
 
@@ -497,3 +511,67 @@ int	    n;
   }
 }
 
+void XawDrawText(w, gc, x, y, item, n)
+Widget	    w;
+GC	    gc;
+int	    x, y;
+XTextItem   *item;
+int	    n;
+{
+  DrawingAreaWidget daw = (DrawingAreaWidget)w;
+
+  if (XtIsRealized(w)) {
+    if (daw->core.visible)
+      XDrawText(XtDisplay(w), XtWindow(w), gc, x, y, item, n);
+    XDrawText(XtDisplay(w), daw->draw.BackingStore, gc, x, y, item, n);
+  }
+}
+
+void XawDrawString16(w, gc, x, y, str, n)
+Widget		w;
+GC		gc;
+int		x, y;
+_Xconst XChar2b	*str;
+int		n;
+{
+  DrawingAreaWidget daw = (DrawingAreaWidget)w;
+
+  if (XtIsRealized(w)) {
+    if (daw->core.visible)
+      XDrawString16(XtDisplay(w), XtWindow(w), gc, x, y, str, n);
+    XDrawString16(XtDisplay(w), daw->draw.BackingStore, gc, x, y, str, n);
+  }
+}
+
+void XawDrawImageString16(w, gc, x, y, str, n)
+Widget		w;
+GC		gc;
+int		x, y;
+_Xconst XChar2b	*str;
+int		n;
+{
+  DrawingAreaWidget daw = (DrawingAreaWidget)w;
+
+  if (XtIsRealized(w)) {
+    if (daw->core.visible)
+      XDrawImageString16(XtDisplay(w), XtWindow(w), gc, x, y, str, n);
+    XDrawImageString16(XtDisplay(w), daw->draw.BackingStore, gc, x, y, str, n);
+  }
+}
+
+void XawDrawText16(w, gc, x, y, item, n)
+Widget	    w;
+GC	    gc;
+int	    x, y;
+XTextItem16 *item;
+int	    n;
+{
+  DrawingAreaWidget daw = (DrawingAreaWidget)w;
+
+  if (XtIsRealized(w)) {
+    if (daw->core.visible)
+      XDrawText16(XtDisplay(w), XtWindow(w), gc, x, y, item, n);
+    XDrawText16(XtDisplay(w), daw->draw.BackingStore, gc, x, y, item, n);
+  }
+}
+
