@@ -1,10 +1,14 @@
-/* $TOG: Label.c /main/98 1998/02/06 12:46:24 kaleb $ */
+/* $Xorg: Label.c,v 1.5 2001/02/09 02:03:43 xorgcvs Exp $ */
 
 /*************************************************************************
 
 Copyright 1987, 1988, 1994, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -41,7 +45,7 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
-This file contains EXTENSIVE modifications for XawPlus, Roland Krause 2000
+This file contains modifications for XawPlus, Roland Krause 2002
 
 ****************************************************************************/
 
@@ -51,8 +55,8 @@ This file contains EXTENSIVE modifications for XawPlus, Roland Krause 2000
  */
 
 #include <stdio.h>
-#include <ctype.h>
 #include <stdlib.h>
+#include "UTF8.h"
 
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
@@ -186,201 +190,6 @@ WidgetClass labelWidgetClass = (WidgetClass)&labelClassRec;
  *
  ****************************************************************/
 
-/* String functions, that works with 16 bit characters.
- * These functions are used to handle strings with 16 bit encoding.
- */
-
-typedef unsigned short char16;	/* The data type used for 16 bit character strings */
-
-/* Determine the length of the string in char16's
- */
-static int str16len(str16)
-char16 *str16;
-{
-   int len = 0;
-
-   while (*str16++) len++;
-   return len;
-}
-
-/* Search the character ch in str16 and return a
- * pointer to this character. Otherwise return NULL.
- */
-static char16 *str16chr(str16, ch)
-char16 *str16;
-char ch;
-{
-   char *p;
-
-   while (*str16)
-   {
-     p = (char *)str16;
-     if ((*p++ == '\0') && (*p == ch)) return(str16);
-     str16++;
-   }
-   return NULL;
-}
-
-/* Copy the string from src to dest.
- */
-static void str16cpy(dest, src)
-char16 *dest, *src;
-{
-   while (*src) *dest++ = *src++;
-   *dest = 0;		/* To terminate the string */
-}
-
-/* Copy n characters from src to dest or
- * until the end of src.
- */
-static void str16ncpy(dest, src, n)
-char16 *dest, *src;
-size_t n;
-{
-   while ((n > 0) && *src)
-   {
-     *dest++ = *src++;
-     n--;
-   }
-   *dest = 0;	/* We always terminate the string here */
-}
-
-/* Copy the content of src behind dest.
- * src and dest has to be null terminated !
- */
-static void str16cat(dest, src)
-char16 *dest, *src;
-{
-   while (*dest) dest++;	  /* search the end of the string */
-   while (*src) *dest++ = *src++; /* copy the other behind */
-   *dest = 0;			  /* and terminate the string */
-}
-
-/* Determine the numer of bytes of the
- * multibyte character pointed to by str.
- * We don't check if the multibyte code is
- * valid !
- */
-static int mbCharLen(str)
-char *str;
-{
-   unsigned char c = (unsigned char)*str;
-   unsigned char mask = 0x80;
-   int count = 0;
-
-   if (c)
-   {
-     while (c & mask)
-     {
-        count++;
-        mask >>= 1;
-     }
-
-     switch (count)
-     {
-	case 0:	 return 1;	/* Character is one byte long */
-	case 1:  return 0;	/* No multibyte string or wrong position */
-	default: return count;
-     }
-   }
-   return 0;	/* End of string reached */
-}
-
-/* Determine the string length in characters (not in byte!)
- * We don't count the trailing null byte here.
- */
-static int mbStrLen(str)
-char *str;
-{
-   char *p = str;
-   int clen, len = 0;
-
-   while ((clen = mbCharLen(p)) > 0)
-   {
-      len++;
-      p += clen;
-   }
-   return len;
-}
-
-/* Conversion of an UTF8 coded string into UCS2/UNICODE.
- * If the encoding of the character is not representable
- * in two bytes, the tilde sign ~ is written into the
- * result string at this position.
- * For an illegal UTF8 code an asterix * is stored in
- * the result string.
- *
- * This function allocates memory for the result, which
- * has to be released by the caller.
- */
-
-static char16 *UTF8toUCS2(str)
-char *str;
-{
-   char16 *str16, *p16, testINTEL = 1, c16;
-   int    clen, cInString;
-   char   *p;
-
-  /* In the first step we try to determine the string
-   * length in characters.
-   */
-   cInString = mbStrLen(str);
-   cInString++;		/* For the terminating null */
-
-   /* Now we need memory for our conversion result */
-
-   str16 = (char16 *)XtMalloc(cInString * sizeof(char16));
-   if (str16)
-   {
-     /* Start the conversion: Determine the number of bytes
-      * for the next character, decode it and store the
-      * result in our result string
-      */
-      p   = str;
-      p16 = str16;
-      while ((clen = mbCharLen(p)) > 0)
-      {
-	 switch (clen)
-	 {
-	    case 1: *p16 = (char16)*p;
-		    break;
-
-	    case 2: c16 = (p[1] & 0x3f) + ((p[0] & 0x1f) << 6);
-		    *p16 = ((c16 > 0x7f) && ((p[1] & 0xC0) == 0x80)) ? c16 : '*';
-		    break;
-
-	    case 3: c16 = (p[2] & 0x3f) + ((p[1] & 0x3f) << 6) + ((p[0] & 0xf) << 12);
-		    *p16 = ((c16 > 0x7ff) && ((p[1] & 0xC0) == 0x80) && ((p[2] & 0xC0) == 0x80)) ? c16 : '*';
-		    break;
-
-	    default: *p16 = '~';	/* character code is greater than 0xffff */
-	 }
-	 p16++;				/* Jump to the next character */
-	 p += clen;
-      }
-      *p16 = 0;				/* String termination */
-
-      /* Swap the bytes, if we are on a machine with an INTEL architecture */
-
-      if (*((char *)&testINTEL))
-      {
-	 char *src, *dest, c;
-
-	 src = dest = (char *)str16;
-	 src++;
-	 while (*src || *dest)
-	 {
-	    c = *dest; *dest = *src; *src = c;
-	    src += 2; dest += 2;
-	 }
-      }
-      return str16;
-   }
-   return NULL;
-}
-
-/* ----------------------------------------------------------------------------- */
-
 /* Calculate width and height of the label for 8 bit characters
  */
 static void SetWidthAndHeight8Bit(lw)
@@ -679,17 +488,20 @@ static void Initialize(request, new)
 {
     LabelWidget lw = (LabelWidget)new;
 
-    if (lw->label.encoding)	/* Wide character encoding */
+    if (lw->label.label == NULL)
     {
-      if (lw->label.label == NULL) lw->label.label = XtNewString(lw->core.name);
-      else lw->label.label = (char *)UTF8toUCS2(lw->label.label);
+       lw->label.label = XtNewString(lw->core.name);
+       lw->label.encoding = FALSE;
     }
-    else	/* 8 bit character or multibyte character encoding */
+    else
     {
-      if (lw->label.label == NULL) lw->label.label = XtNewString(lw->core.name);
-      else lw->label.label = XtNewString(lw->label.label);
-    }
+       if (lw->label.encoding)	/* Wide character encoding */
+	  lw->label.label = (char *)UTF8toUCS2(lw->label.label);
 
+       /* 8 bit character or multibyte character encoding */
+
+       else lw->label.label = XtNewString(lw->label.label);
+    }
     GetnormalGC(lw);
     GetgrayGCs(lw);
 
