@@ -1,9 +1,13 @@
-/* $TOG: List.c /main/38 1998/02/11 14:53:23 kaleb $ */
+/* $Xorg: List.c,v 1.4 2001/02/09 02:03:44 xorgcvs Exp $ */
 
 /*
 Copyright 1989, 1994, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -30,7 +34,7 @@ in this Software without prior written authorization from The Open Group.
  *	By:		Chris D. Peterson
  *                      MIT X Consortium
  *
- * This file contains modifications for XawPlus, Roland Krause 2000
+ * This file contains modifications for XawPlus, Roland Krause 2002
  */
 
 #include <stdio.h>
@@ -1222,7 +1226,7 @@ Widget w;
 
     if (ret_val) {
       ret_val->list_index = lw->list.highlight;
-      if (ret_val->list_index == XAW_LIST_NONE) *ret_val->string = '\0';
+      if (ret_val->list_index == XAW_LIST_NONE) ret_val->string = "";
       else ret_val->string = lw->list.iconList ? lw->list.iconList[ret_val->list_index].string
 					       : lw->list.list[ret_val->list_index];
     }
