@@ -1,9 +1,13 @@
-/* $TOG: SmeBSB.c /main/23 1998/06/24 11:53:39 kaleb $ */
+/* $Xorg: SmeBSB.c,v 1.5 2001/02/09 02:03:45 xorgcvs Exp $ */
 
 /*
 Copyright 1989, 1994, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -29,29 +33,27 @@ in this Software without prior written authorization from The Open Group.
  *          MIT X Consortium 
  *          kit@expo.lcs.mit.edu
  *
- * This file contains modifications for XawPlus, Roland Krause 2000
+ * This file contains modifications for XawPlus, Roland Krause 2002
  */
 
+#include <stdio.h>
+#include <stdlib.h>
+#include "UTF8.h"
+
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/Xos.h>
-
-/* needed for abs() */
-#ifdef X_NOT_STDC_ENV
-extern int abs();
-#else
-#include <stdlib.h>
-#endif
-
-#include <X11/Xmu/Drawing.h>
-
 #include <X11/XawPlus/XawInit.h>
 #include <X11/XawPlus/SimpleMenu.h>
 #include <X11/XawPlus/SmeBSBP.h>
 #include <X11/XawPlus/Cardinals.h>
 #include <X11/XawPlus/StrToPmap.h>
+#include <X11/Xmu/Drawing.h>
+
+#ifdef X_NOT_STDC_ENV
+extern int abs();
+#endif
 
-#include <stdio.h>
 
 #define ONE_HUNDRED 100
 
@@ -82,6 +84,8 @@ static XtResource resources[] = {
      offset(font), XtRString, XtDefaultFont},
   {XtNfontSet,  XtCFontSet, XtRFontSet, sizeof(XFontSet ),
      offset(fontset),XtRString, XtDefaultFontSet},
+  {XtNencoding, XtCEncoding, XtRUnsignedChar, sizeof(unsigned char),
+     offset(encoding), XtRImmediate, (XtPointer)XawTextEncoding8bit},
 };   
 #undef offset
 
@@ -193,20 +197,24 @@ Cardinal *num_args;
 {
     SmeBSBObject entry = (SmeBSBObject) new;
 
-    if (entry->sme_bsb.label == NULL) 
-	entry->sme_bsb.label = XtName(new);
+    if (entry->sme_bsb.label == NULL)
+    {
+       entry->sme_bsb.label = XtName(new);
+       entry->sme_bsb.encoding = FALSE;
+    }
     else
-	entry->sme_bsb.label = XtNewString( entry->sme_bsb.label );
+    {
+       if (entry->sme_bsb.encoding)	/* Wide character encoding */
+	 entry->sme_bsb.label = (char *)UTF8toUCS2(entry->sme_bsb.label);
 
-    GetDefaultSize(new, &(entry->rectangle.width), &(entry->rectangle.height));
+       /* 8 bit character or multibyte character encoding */
+
+       else entry->sme_bsb.label = XtNewString(entry->sme_bsb.label);
+    }
     CreateGCs(new);
-
-    entry->sme_bsb.left_bitmap_width = entry->sme_bsb.left_bitmap_height = 0;
-    entry->sme_bsb.right_bitmap_width = entry->sme_bsb.right_bitmap_height = 0;
-    entry->sme_bsb.left_bitmap_depth = entry->sme_bsb.right_bitmap_depth = 1;
-
     GetBitmapInfo(new, TRUE);	/* Left Bitmap Info */
     GetBitmapInfo(new, FALSE);	/* Right Bitmap Info */
+    GetDefaultSize(new, &(entry->rectangle.width), &(entry->rectangle.height));
 }
 
 /*      Function Name: Destroy
@@ -233,6 +241,109 @@ Widget w;
  *      Returns: none.
  */
 
+static void DrawString16(w, gc, x, y, label, len)
+Widget	w;
+GC	gc;
+int	x, y;
+char	*label;
+int	len;
+{
+  SmeBSBObject entry = (SmeBSBObject)w;
+
+  if (XtIsSensitive(w) && XtIsSensitive(XtParent(w)))
+  {
+    /* Draw the string in normal or revert style */
+
+    XDrawString16(XtDisplayOfObject(w), XtWindowOfObject(w), gc, x, y, (XChar2b *)label, len);
+  }
+  else
+  {
+    /* Draw the string in the insensitive style */
+
+    XDrawString16(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.lgray_gc,
+	          x+1, y+1, (XChar2b *)label, len);
+    XDrawString16(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.gray_gc,
+	          x, y, (XChar2b *)label, len);
+  }
+}
+
+/* ------------------------------------------------------------------------- */
+
+static void DrawStringInt(w, gc, x, y, label, len)
+Widget	w;
+GC	gc;
+int	x, y;
+char	*label;
+int	len;
+{
+  SmeBSBObject entry = (SmeBSBObject)w;
+
+  if (XtIsSensitive(w) && XtIsSensitive(XtParent(w)))
+  {
+    /* Draw the multi byte string in normal or revert style */
+
+    XmbDrawString(XtDisplayOfObject(w), XtWindowOfObject(w),
+		  entry->sme_bsb.fontset, gc, x, y, label, len);
+  }
+  else
+  {
+    /* Draw the multi byte string in the insensitive style */
+
+    XmbDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.fontset,
+		  entry->sme_bsb.lgray_gc, x+1, y+1, label, len);
+    XmbDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.fontset,
+		  entry->sme_bsb.gray_gc, x, y, label, len);
+  }
+}
+
+/* ------------------------------------------------------------------------- */
+
+static void DrawString8(w, gc, x, y, label, len)
+Widget	w;
+GC	gc;
+int	x, y;
+char	*label;
+int	len;
+{
+  SmeBSBObject entry = (SmeBSBObject)w;
+
+  if (XtIsSensitive(w) && XtIsSensitive(XtParent(w)))
+  {
+    /* Draw the string in normal or revert style */
+
+    XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), gc, x, y, label, len);
+  }
+  else
+  {
+    /* Draw the string in the insensitive style */
+
+    XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.lgray_gc,
+	        x+1, y+1, label, len);
+    XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.gray_gc,
+	        x, y, label, len);
+  }
+}
+
+/* ------------------------------------------------------------------------- */
+
+static int CalcTextWidth(entry, label, len)
+SmeBSBObject	entry;
+char		*label;
+int		len;
+{
+   int width;
+
+   if (entry->sme_bsb.encoding) width = XTextWidth16(entry->sme_bsb.font, (XChar2b *)label, len);
+   else
+   {
+      if (entry->sme.international) width = XmbTextEscapement(entry->sme_bsb.fontset,label,len);
+      else width = XTextWidth(entry->sme_bsb.font, label, len);
+   }
+   return width;
+}
+
+/* ------------------------------------------------------------------------- */
+
 static void Redisplay(w, event, region)
 Widget w;
 XEvent * event;
@@ -241,7 +352,6 @@ Region region;
     GC gc;
     SmeBSBObject entry = (SmeBSBObject) w;
     int		 font_ascent, font_descent, x_loc, y_loc;
-    Boolean	 insensitive;
 
     int	fontset_ascent, fontset_descent;
     XFontSetExtents *ext = XExtentsOfFontSet(entry->sme_bsb.fontset);
@@ -257,7 +367,7 @@ Region region;
     }
     y_loc = entry->rectangle.y;
     x_loc = entry->rectangle.x;
-    
+
     if (XtIsSensitive(w) && XtIsSensitive( XtParent(w) ) ) {
 	if ( w == XawSimpleMenuGetActiveEntry(XtParent(w)) ) {
 	    XFillRectangle(XtDisplayOfObject(w), XtWindowOfObject(w), 
@@ -267,94 +377,68 @@ Region region;
 	    gc = entry->sme_bsb.rev_gc;
 	}
 	else gc = entry->sme_bsb.norm_gc;
-	insensitive = FALSE;
-    }
-    else {
-      gc = entry->sme_bsb.norm_gc;
-      insensitive = TRUE;
     }
+    else gc = entry->sme_bsb.norm_gc;
     
     if (entry->sme_bsb.label != NULL) {
-	int  len    = strlen(entry->sme_bsb.label);
 	char *label = entry->sme_bsb.label;
-	x_loc	    = entry->sme_bsb.left_margin;
+	int  len, width, t_width;
 
+	/* Determine the length of the label string */
 
-	switch(entry->sme_bsb.justify) {
-	    int width, t_width;
+	if (entry->sme_bsb.encoding) len = str16len(label);
+	else len = strlen(label);
 
-	case XtJustifyCenter:
-            if ( entry->sme.international == True ) {
-	        t_width = XmbTextEscapement(entry->sme_bsb.fontset,label,len);
-                width = entry->rectangle.width - (entry->sme_bsb.left_margin +
-					      entry->sme_bsb.right_margin);
-            }
-            else {
-	        t_width = XTextWidth(entry->sme_bsb.font, label, len);
-	        width = entry->rectangle.width - (entry->sme_bsb.left_margin +
-					      entry->sme_bsb.right_margin);
-            }
-	    x_loc += (width - t_width)/2;
-	    break;
-	case XtJustifyRight:
-            if ( entry->sme.international == True ) {
-                t_width = XmbTextEscapement(entry->sme_bsb.fontset,label,len);
-                x_loc = entry->rectangle.width - ( entry->sme_bsb.right_margin
-						 + t_width );
-            }
-            else {
-	        t_width = XTextWidth(entry->sme_bsb.font, label, len);
-	        x_loc = entry->rectangle.width - ( entry->sme_bsb.right_margin
-						 + t_width );
-            }
-	    break;
-	case XtJustifyLeft:
-	default:
-	    break;
+	/* Calculate the position of the string */
+
+	switch(entry->sme_bsb.justify)
+	{
+	   case XtJustifyCenter:
+
+		t_width = CalcTextWidth(entry, label, len);
+		width = entry->rectangle.width - (entry->sme_bsb.left_margin +
+						  entry->sme_bsb.right_margin);
+		x_loc = entry->sme_bsb.left_margin + (width - t_width)/2;
+		break;
+
+	   case XtJustifyRight:
+
+		t_width = CalcTextWidth(entry, label, len);
+		x_loc = entry->rectangle.width - (entry->sme_bsb.right_margin + t_width);
+		break;
+
+	   default:	/* Everything else is XtJustifyLeft */
+
+		x_loc = entry->sme_bsb.left_margin;
+		break;
 	}
-	/* this will center the text in the gadget top-to-bottom */
+	/* Now we draw the string at its position */
 
-        if ( entry->sme.international==True )
+	if (entry->sme_bsb.encoding)	/* Label string with UCS2 code/UNICODE */
 	{
-          y_loc += ((int)entry->rectangle.height - 
-		    (fontset_ascent + fontset_descent)) / 2 + fontset_ascent;
-	  if (insensitive == TRUE)
-	  {
-	    /* Draw the multi byte string in the insensitive style
-	     */
-	    XmbDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.fontset,
-			  entry->sme_bsb.lgray_gc, x_loc+1, y_loc+1, label, len);
-	    XmbDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.fontset,
-			  entry->sme_bsb.gray_gc, x_loc, y_loc, label, len);
-	  }
-	  /* Draw the multi byte string in normal or revert style
-	   */
-          else XmbDrawString(XtDisplayOfObject(w), XtWindowOfObject(w),
-			     entry->sme_bsb.fontset, gc, x_loc, y_loc, label, len);
-        }
-        else
+		y_loc += ((int)entry->rectangle.height - 
+			  (font_ascent + font_descent)) / 2 + font_ascent;
+		DrawString16(w, gc, x_loc, y_loc, label, len);
+	}
+	else				/* Label string with multibyte code */
 	{
-          y_loc += ((int)entry->rectangle.height - 
-		    (font_ascent + font_descent)) / 2 + font_ascent;
-	  if (insensitive == TRUE)
-	  {
-	    /* Draw the string in the insensitive style
-	     */
-	    XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.lgray_gc,
-		        x_loc+1, y_loc+1, label, len);
-	    XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.gray_gc,
-		        x_loc, y_loc, label, len);
-	  }
-	  /* Draw the string in normal or revert style
-	   */
-          else XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), gc,
-		    	   x_loc, y_loc, label, len);
-        }
+	    if (entry->sme.international)
+	    {
+		y_loc += ((int)entry->rectangle.height - 
+			  (fontset_ascent + fontset_descent)) / 2 + fontset_ascent;
+		DrawStringInt(w, gc, x_loc, y_loc, label, len);
+	    }
+	    else			/* Label string with 8 bit code */
+	    {
+		y_loc += ((int)entry->rectangle.height - 
+			  (font_ascent + font_descent)) / 2 + font_ascent;
+		DrawString8(w, gc, x_loc, y_loc, label, len);
+	    }
+	}
     }
     DrawBitmaps(w, gc);
 }
 
-
 /*      Function Name: SetValues
  *      Description: Relayout the menu when one of the resources is changed.
  *      Arguments: current - current state of the widget.
@@ -368,21 +452,51 @@ Widget current, request, new;
 ArgList args;
 Cardinal *num_args;
 {
-    SmeBSBObject entry = (SmeBSBObject) new;
-    SmeBSBObject old_entry = (SmeBSBObject) current;
-    Boolean ret_val = FALSE;
+    SmeBSBObject entry 	   = (SmeBSBObject)new;
+    SmeBSBObject old_entry = (SmeBSBObject)current;
+    Boolean ret_val	   = FALSE,
+	    labelModified  = FALSE;
 
-    if (old_entry->sme_bsb.label != entry->sme_bsb.label) {
-        if (old_entry->sme_bsb.label != XtName( new ) )
-	    XtFree( (char *) old_entry->sme_bsb.label );
+    /* Handle changes of encoding */
 
-	if (entry->sme_bsb.label != XtName(new) ) 
-	    entry->sme_bsb.label = XtNewString( entry->sme_bsb.label );
+    if (old_entry->sme_bsb.encoding != entry->sme_bsb.encoding)
+    {
+       /* If the encoding changes and the label not, the
+	* label string has the wrong format and we have to
+	* modify it. If both changes, the encoding and the label,
+	* in the next step a label in the necessary format is created.
+	*/
+       if (old_entry->sme_bsb.label == entry->sme_bsb.label)
+       {
+	  if (entry->sme_bsb.encoding)
+	    entry->sme_bsb.label = (char *)UTF8toUCS2(entry->sme_bsb.label);
+	  else entry->sme_bsb.label = XtName(new);
+	  if (old_entry->sme_bsb.label != XtName(new)) XtFree(old_entry->sme_bsb.label);
+       }
+       ret_val = labelModified = TRUE;
+    }
+    /* Handle changes of the label string */
 
-	ret_val = True;
+    if (!labelModified && (old_entry->sme_bsb.label != entry->sme_bsb.label))
+    {
+	/* Wide character encoding */
+
+	if (entry->sme_bsb.encoding)
+	{
+	   entry->sme_bsb.label = (char *)UTF8toUCS2(entry->sme_bsb.label);
+	   XtFree(old_entry->sme_bsb.label);
+	}
+	/* 8 bit character or multibyte character encoding */
+
+	else if (entry->sme_bsb.label != XtName(new))
+	{
+	   entry->sme_bsb.label = XtNewString(entry->sme_bsb.label);
+	   XtFree(old_entry->sme_bsb.label);
+	}
+	ret_val = TRUE;
     }
 
-    if (entry->rectangle.sensitive != old_entry->rectangle.sensitive )
+    if (entry->rectangle.sensitive != old_entry->rectangle.sensitive)
 	ret_val = TRUE;
 
     if ((entry->sme_bsb.left_bitmap != old_entry->sme_bsb.left_bitmap) ||
@@ -501,37 +615,45 @@ Widget w;
 
 static void GetDefaultSize(w, width, height) 
 Widget w;
-Dimension * width, * height;
+Dimension *width, *height;
 {
-    SmeBSBObject entry = (SmeBSBObject) w;
+    SmeBSBObject entry = (SmeBSBObject)w;
 
-    if ( entry->sme.international == True ) {
-        XFontSetExtents *ext = XExtentsOfFontSet(entry->sme_bsb.fontset);
-        if (entry->sme_bsb.label == NULL) 
-	    *width = 0;
-        else
-	    *width = XmbTextEscapement(entry->sme_bsb.fontset, entry->sme_bsb.label,
-			    strlen(entry->sme_bsb.label));
-        *width += entry->sme_bsb.left_margin + entry->sme_bsb.right_margin;
-        *height = ext->max_ink_extent.height;
-        *height = ((int)*height * ( ONE_HUNDRED + 
-			        entry->sme_bsb.vert_space )) / ONE_HUNDRED;
-    }
-    else {
-        if (entry->sme_bsb.label == NULL) 
-	    *width = 0;
-        else
-	    *width = XTextWidth(entry->sme_bsb.font, entry->sme_bsb.label,
-			    strlen(entry->sme_bsb.label));
+    /* Initialize width and height */
 
-        *width += entry->sme_bsb.left_margin + entry->sme_bsb.right_margin;
-    
-        *height = (entry->sme_bsb.font->max_bounds.ascent +
+    *width = entry->sme_bsb.left_margin + entry->sme_bsb.right_margin;
+    *height = (entry->sme_bsb.font->max_bounds.ascent +
 	       entry->sme_bsb.font->max_bounds.descent);
 
-        *height = ((int)*height * ( ONE_HUNDRED + 
-			        entry->sme_bsb.vert_space )) / ONE_HUNDRED;
+    /* Now calculate the final values adding the string width and
+     * the font height to the current width and height.
+     */
+    if (entry->sme_bsb.encoding)
+      *width += XTextWidth16(entry->sme_bsb.font, (XChar2b *)entry->sme_bsb.label,
+			     str16len(entry->sme_bsb.label));
+    else
+    {
+	if (entry->sme.international)
+	{
+	   XFontSetExtents *ext = XExtentsOfFontSet(entry->sme_bsb.fontset);
+
+	   *width += XmbTextEscapement(entry->sme_bsb.fontset, entry->sme_bsb.label,
+				       strlen(entry->sme_bsb.label));
+	   *height = ext->max_ink_extent.height;
+	}
+	else *width += XTextWidth(entry->sme_bsb.font, entry->sme_bsb.label,
+				  strlen(entry->sme_bsb.label));
     }
+    /* Let us see, if the left or the right bitmap is higher than the
+     * font height and if so, use this value instead of the font height.
+     */
+    if (entry->sme_bsb.right_bitmap_height > *height)
+      *height = entry->sme_bsb.right_bitmap_height;
+
+    if (entry->sme_bsb.left_bitmap_height > *height)
+      *height = entry->sme_bsb.left_bitmap_height;
+
+    *height = ((int)*height * (ONE_HUNDRED + entry->sme_bsb.vert_space ))/ONE_HUNDRED;
 }
 
 /*      Function Name: DrawBitmaps
@@ -545,15 +667,13 @@ static void DrawBitmaps(w, gc)
 Widget w;
 GC gc;
 {
-    int x_loc, y_loc;
-    SmeBSBObject entry = (SmeBSBObject) w;
+  int x_loc, y_loc;
+  SmeBSBObject entry = (SmeBSBObject) w;
     
-    if ( (entry->sme_bsb.left_bitmap == None) && 
-	 (entry->sme_bsb.right_bitmap == None) ) return;
+  if ((entry->sme_bsb.left_bitmap == None) && 
+      (entry->sme_bsb.right_bitmap == None)) return;
 
-/*
- * Draw Left Bitmap or Pixmap
- */
+  /* Draw Left Bitmap or Pixmap */
 
   if (entry->sme_bsb.left_bitmap != None) {
     x_loc = (int)(entry->sme_bsb.left_margin - entry->sme_bsb.left_bitmap_width) / 2;
@@ -577,11 +697,7 @@ GC gc;
       XSetClipMask(XtDisplayOfObject(w), gc, None);
     }
   }
-
-/*
- * Draw Right Bitmap or Pixmap
- */
-
+  /* Draw Right Bitmap or Pixmap */
 
   if (entry->sme_bsb.right_bitmap != None) {
     x_loc = entry->rectangle.width - (int)(entry->sme_bsb.right_margin +
@@ -620,15 +736,17 @@ static void GetBitmapInfo(w, is_left)
 Widget w;
 Boolean is_left;
 {
-    SmeBSBObject entry = (SmeBSBObject) w;    
-    unsigned int depth, bw;
-    Window root;
-    int x, y;
-    unsigned int width, height;
-    char buf[BUFSIZ];
+   SmeBSBObject entry = (SmeBSBObject) w;    
+   unsigned int depth, bw;
+   Window root;
+   int x, y;
+   unsigned int width, height;
+   char buf[BUFSIZ];
     
-    if (is_left) {
-      if (entry->sme_bsb.left_bitmap != None) {
+   if (is_left)
+   {
+     if (entry->sme_bsb.left_bitmap != None)
+     {
 	if (!XGetGeometry(XtDisplayOfObject(w), entry->sme_bsb.left_bitmap, &root, 
 			  &x, &y, &width, &height, &bw, &depth)) {
 	   sprintf(buf, "SmeBSB Object: %s \"%s\".",
@@ -639,8 +757,15 @@ Boolean is_left;
 	entry->sme_bsb.left_bitmap_height = (Dimension) height;
         entry->sme_bsb.left_bitmap_depth = depth;
      }
+     else /* No left bitmap */
+     {
+	entry->sme_bsb.left_bitmap_width  = 0;
+	entry->sme_bsb.left_bitmap_height = 0;
+	entry->sme_bsb.left_bitmap_depth  = 1;
+     }
    }
-   else if (entry->sme_bsb.right_bitmap != None) {
+   else if (entry->sme_bsb.right_bitmap != None)
+   {
      if (!XGetGeometry(XtDisplayOfObject(w), entry->sme_bsb.right_bitmap, &root,
 			&x, &y, &width, &height, &bw, &depth)) {
 	sprintf(buf, "SmeBSB Object: %s \"%s\".",
@@ -651,6 +776,12 @@ Boolean is_left;
      entry->sme_bsb.right_bitmap_height = (Dimension) height;
      entry->sme_bsb.right_bitmap_depth = depth;
    }
+   else /* No right bitmap */
+   {
+      entry->sme_bsb.right_bitmap_width  = 0;
+      entry->sme_bsb.right_bitmap_height = 0;
+      entry->sme_bsb.right_bitmap_depth  = 1;
+   }
 }      
 
 /*      Function Name: CreateGCs
