@@ -1,11 +1,15 @@
 /*************************************************************************
 
-  $TOG: PanedP.h /main/8 1998/02/11 14:53:55 kaleb $
+  $Xorg: PanedP.h,v 1.4 2001/02/09 02:03:45 xorgcvs Exp $
 
 
 Copyright 1987, 1988, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -55,7 +59,7 @@ SOFTWARE.
  *          MIT X Consortium
  *          kit@expo.lcs.mit.edu
  *
- * This file contains modifications for XawPlus, Roland Krause 2000
+ * This file contains modifications for XawPlus, Roland Krause 2002
  */
 
 #ifndef _XawPanedP_h
