@@ -1,9 +1,13 @@
 /*
- * $TOG: Panner.h /main/24 1998/02/11 14:54:19 kaleb $
+ * $Xorg: Panner.h,v 1.5 2001/02/09 02:03:45 xorgcvs Exp $
  *
 Copyright 1989, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -27,6 +31,7 @@ in this Software without prior written authorization from The Open Group.
 #ifndef _XawPanner_h
 #define _XawPanner_h
 
+#include <X11/Intrinsic.h>
 #include <X11/XawPlus/Reports.h>
 
 /*****************************************************************************
