@@ -1,9 +1,13 @@
 /*
- * $TOG: Sme.h /main/8 1998/02/11 14:55:35 kaleb $
+ * $Xorg: Sme.h,v 1.5 2001/02/09 02:03:45 xorgcvs Exp $
  *
 Copyright 1989, 1994, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -38,6 +42,7 @@ in this Software without prior written authorization from The Open Group.
 #ifndef _Sme_h
 #define _Sme_h
 
+#include <X11/Intrinsic.h>
 #include <X11/RectObj.h>
 
 /****************************************************************
