@@ -1,9 +1,13 @@
 /*
- * $TOG: SmeBSBP.h /main/10 1998/02/11 14:55:01 kaleb $
+ * $Xorg: SmeBSBP.h,v 1.4 2001/02/09 02:03:45 xorgcvs Exp $
  *
 Copyright 1989, 1994, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -21,7 +25,7 @@ in this Software without prior written authorization from The Open Group.
  *
  * Author:  Chris D. Peterson, MIT X Consortium
  * 
- * This file contains modifications for XawPlus, Roland Krause 2000
+ * This file contains modifications for XawPlus, Roland Krause 2002
  */
 
 /* 
@@ -75,6 +79,7 @@ typedef struct {
   Pixel     foreground;		/* foreground color. */
   XFontStruct *font;		/* The font to show the label */
   XFontSet  fontset;		/* .. or the font set */
+  unsigned char encoding;	/* Type of encoding, 8 bit or 16 bit in UTF8 */
   XtJustify justify;		/* Justification for the label. */
 
 /* private resources. */
